@@ -5,4 +5,4 @@
 # LICENSE file in the root directory of this source tree.
 
 # import handlers
-from grader_service.handlers import assignment, grading, lectures, submissions, git, permission
\ No newline at end of file
+from grader_service.handlers import assignment, grading, lectures, submissions, git, permission, health, config
